
try:
    from psyco.classes import *
except ImportError:
    pass

from math import sin,cos
from random import random, choice

import pygame
from pygame.locals import *

from pygext.gl.all import *
from pygext.gl.shapes.simple import rect
from pygext.math.vecalg import *
from pygext.lazy import Random

from steamer.world import *

try:
    Set = set
except NameError:
    from sets import Set

class TreeEmitter(CircleEmitter):
    tanget = False
    align_to_node = False
    radius = 10

    direction = 0
    angle = 140
    velocity = Random(40,120)
    num_particles = 30
    num_emits = 1
    scale = Random(0.4,0.8)
    rotation = Random(360)
    rotation_delta = Random(-50,50)
    life = Random(0.5,1.5)
    fade_time = 0.1

class Tree(Entity):
    layer = "objects"
    
    def init(self, slope):
        self.slope = slope
        self.add_collnode("tree", 16, y=-22)

    def fall(self, dir):
        self.place("objects")
        self.clear_collnodes()
        ta = self.slope + dir * 90
        ca = self.angle
        if ta > 180:
            ta -= 360
        if ca > 180:
            ca -= 360
        delta = ta-ca
        dirt = CallFunc(World.dirt_system.new_emitter, TreeEmitter, x=self.x+dir*10, y=self.y)
        self.do(RotateDelta(delta, secs=0.3) + dirt + CallFunc(self.restatic))

    def restatic(self):
        self.place("static trees")

class House(Entity):
    layer = "objects"

    def init(self):
        self.add_collnode("house", 16, y=-16)

class BlimpExplosionEmitter(RectEmitter):
    rect = [-50,-20,50,20]

    tangent = False
    align_to_node = False
    direction = 0
    angle = 360
    num_particles = 40
    num_emits = 1
    color = (255,255,255,Random(150,200))
    velocity = Random(100,250)
    scale = Random(1.0,2.0)
    scale_delta = -0.5
    life = Random(0.5,1.5)
    fade_time = 1

class CannonExplosionEmitter(CircleEmitter):
    radius = 10
    
    tangent = False
    align_to_node = False
    direction = 0
    angle = 120
    num_particles = 40
    num_emits = 1
    color = (255,255,255,Random(150,200))
    velocity = Random(100,250)
    scale = Random(1.0,2.0)
    scale_delta = -0.5
    life = Random(0.5,1.5)
    fade_time = 1
    

class Blimp(Entity):
    layer = "objects"
    image = "gfx/blimp.png"

    def init(self):
        self.add_collnode("blimp", 20, x=-40)
        self.add_collnode("blimp", 20, x=0)
        self.add_collnode("blimp", 20, x=40)
        self.move = Move(-80,0).do(self)
        self.dead = False

    def boom(self):
        if self.dead:
            return
        self.dead = True
        self.clear_collnodes()
        World.fire_system.new_emitter(BlimpExplosionEmitter, self)
        World.smoke_system.new_emitter(CannonEmitter, self)
        self.do(AlphaFade(0,secs=1.5) + Delete)
        self.do(Rotate(-10))
        self.move.set_velocity(-80,40)

    def fall(self, e):
        pass

class Pad(Entity):
    image = "gfx/pad.png"

class Cannon(EntityNode):
    image = "gfx/cannon_wheels.png"
    layer = "objects"

    def init(self):
        self.turret = EntityNode("gfx/cannon_barrel.png", hotspot=(0.5,0.75))
        self.turret.attach_to(self)
        self.turret.set(y = -3)

        self.node = Entity()
        self.node.attach_to(self.turret).set(x=0,y=-30)

        self.add_collnode("cannon", 14)
        self.dead = False

        self.recharging = False
        self.turn = Rotate(0).do(self.turret)

    def boom(self):
        if self.dead:
            return
        self.dead = True
        World.fire_system.new_emitter(CannonExplosionEmitter, self)
        World.smoke_system.new_emitter(CannonEmitter, self)
        self.do(AlphaFade(0,secs=0.5) + Delete)
    

    def tick(self, ship):
        if self.dead:
            return
        shipx = ship.x + ship.move.vx * 0.7
        shipy = ship.y + ship.move.vy * 0.7
        dx = shipx-self.x
        if abs(dx) > 600:
            self.turn.set_velocity(0)
            return
        a = self.turret.angle
        if a > 180:
            a -= 360
        ta = direction(vec(dx,shipy-self.y))
        if ta > 180:
            ta -= 360
        ta *= 0.99 ** abs(dx/200.0)
        if ta > 70:
            ta = 70
        if ta < -70:
            ta = -70
        if abs(a-ta) > 15:
            self.turn.set_velocity(cmp(ta,a)*15)
        elif abs(a) > 5:
            self.shoot()
            self.turn.set_velocity(0)

    def shoot(self):
        if self.recharging:
            return
        self.node.angle = self.turret.angle
        b = Bullet(node=self.node)
        self.recharging = True
        self.do(Delay(4) + SetAttr(recharging=False))
        World.smoke_system.new_emitter(CannonEmitter, self.node)

def y_generator(startx=0, starty=0, startdy=0, maxampl=2, maxslope=100, limits=(10,450), plateus=None, xmul=1):
        y = starty
        dy = startdy
        dyy = 0
        la,lb = limits
        x = startx
        while 1:
            if plateus and int(x) in plateus:
                dy *= 0.8
            else:
                if dy > maxampl:
                    dyy -= random()
                elif dy < -maxampl:
                    dyy += random()
                else:
                    dyy += random()*2 - 1.01
                if dyy > maxslope:
                    dyy = maxslope
                elif dyy < -maxslope:
                    dyy = -maxslope
                if y < la+50:
                    dyy += random() * 0.5                
                elif y > lb-50:
                    dyy -= random() * 0.5
                elif y > lb-150:
                    dyy -= random() * 0.1
                dy += dyy * 0.005
                dy *= 0.98
            y += dy
            if (y < la and dy < 0) or (y > lb and dy > 0):
                dy = -dy * 0.5
                dyy = 0
            yield y
            x += xmul
    
def col():
    return int(random()*100+150)


class Land(object):
    def __init__(self, level):
        self.level = level
        num = level.num_segments
        self.segments = []

        self.pads = []

        self.cannons = []
        numcannons = level.num_cannons
        
        y = 200
        dy = 0
        self.length = length = num*128
        plateus = Set()
        plateus |= Set(range(50,120))
        plateus |= Set(range(length-120,length-50))
        pattern = PatternImage("gfx/dirt128x128.png", 128,500)

        l = (length-200) / float(numcannons+1)
        x = l + 100
        for i in range(numcannons):
            x = int(x)
            c = Cannon().set(x=x)
            self.cannons.append(c)
            plateus |= Set(range(x-15,x+15))
            x += l
       
        for x in range(num):
            l = LandSegment(offset=x*128, width=128,starty=y, startdy=dy, plateus=plateus)
            l.generate()
            y = l.y
            dy = l.dy            
            e = EntityNode(l.bmp).set(left=x*128, bottom=640-int(l.miny)).place("land")
            if l.miny > 0:
                e2 = Entity(pattern)
                e2.set(y=e.get_bounding_rect().height)
                e2.attach_to(e)            
            self.segments.append(l)
        x = 0
        decors = [("gfx/dirtdetail%i.png" % i) for i in range(1,9)]
        while x < length:
            x += random() * 200 + 50
            x = int(x)
            h = self.get_min(x-70, x+70)
            e = Entity(choice(decors))
            e.top = 600 - h + random() * 300 + 10
            e.centerx = x
            e.place("land detail")

        # trees
        l = length
        trees = [("gfx/tree%i.png" % i) for i in range(1,3)]
        x = 120
        while x < length-250:
            tx = int(x+random()*200)
            if int(tx) in plateus:
                continue
            h = self.get_min(tx-3,tx+3)
            dh = self.get_h(tx+5) - self.get_h(tx-5)
            v = vec(10,dh)
            a = direction(ortho(v))
            if a > 180:
                a -= 360            
            y = 600 - h
            e = Tree(choice(trees), hotspot=(0.5,1), slope=-a)
            e.set(
                centerx = tx,
                bottom = y+3,
                angle = -a * 0.3
                )
            x += 150 + random()*250

        pad = Pad("gfx/pad.png").set(
            left=50,
            bottom=605-self.get_h(70),
            ).place("land detail")
        self.pads.append(pad)

        pad = Pad("gfx/pad.png").set(
            left=length-120,
            bottom=605-self.get_h(length-100),
            ).place("land detail")
        self.pads.append(pad)

        for c in self.cannons:
            h = self.get_h(c.x)
            c.bottom = 600-h

        bd = level.blimp_density

        if bd > 0:
            span = 4000.0/bd
            x = 400 + random() * 400
            while x < length:
                b = Blimp().set(
                    x=x,
                    y=-10 + random() * -190,
                    color = (col(),col(),col(),255)
                    )
                x += span + random() * span

            t = span/80.0
            delay = t + random()*t
            act = Delay(delay) + CallFunc(self.create_blimp)
            act.do()

    def create_blimp(self):
        bd = self.level.blimp_density
        span = 4000.0/bd
        t = span/80.0
        
        b = Blimp().set(
            x = self.length+100,
                y=-10 + random() * -190,
                color = (col(),col(),col(),255)
            )
        delay = t + random()*t
        act = Delay(delay) + CallFunc(self.create_blimp)
        act.do()

    def get_h(self, x):
        i = x // 128
        if i < 0 or i >= len(self.segments):
            return 0
        s = self.segments[i]
        i = x % 128
        return s.heights[i]-20

    def get_hs(self, xlist):
        return map(self.get_h, xlist)

    def get_min(self, start, end):
        return min((self.get_h(x) for x in range(start,end)))
            
        

class LandSegment(object):
    def __init__(self, offset=0, width=256, starty=200, startdy=0, plateus=None):
        self.plateus = plateus
        self.offset = offset
        self.heights = []
        self.miny = starty
        self.maxy = starty
        self.starty = starty
        self.startdy = startdy
        self.width = width

    def generate(self):
        y = self.starty
        dy = self.startdy
        offset = self.offset
        x = 0        
        gen = y_generator(offset,y,dy, maxampl=2, maxslope=200, limits=(180,600),plateus=self.plateus)
        gen2 = y_generator(offset,0,0,limits=(0,500), plateus=self.plateus,xmul=0.25)
        while x < self.width:
            y = gen.next()
            for i in range(4):
                y2 = gen2.next()
            ry = y + y2*0.05
            self.heights.append(ry)
            if ry < self.miny:
                self.miny = ry
            if ry > self.maxy:
                self.maxy = ry
            x += 1
        h = int(self.maxy - self.miny) + 40
        s = pygame.Surface((self.width+2, h+2), SRCALPHA, 32)
        s.fill((0,0,0,0))
        ground = Bitmap._load_surf("gfx/dirt128x128.png")        
        s.blit(ground, (0,0))
        if h > 128:
            s.blit(ground, (0,128))
        v = s.map_rgb((0,0,0,0))
        px = pygame.surfarray.pixels2d(s)
        for x,y in enumerate(self.heights):
            a = int(self.maxy-y)+20
            px[x+1,0:a] = v
        px[0,:] = px[1,:]
        px[-1,:] = px[-2,:]
        px[:,-1] = px[:,-2]
        del px
        apx = pygame.surfarray.pixels_alpha(s)
        for x,y in enumerate(self.heights):
            a = int(self.maxy-y)+20
            apx[x+1,a:a+5] = range(0,150,30)
        del apx
        self.y = y
        self.dy = dy
        skip = 0
        grass = [Bitmap._load_surf("gfx/grass%i.png" % i) for i in range(1,4)]
        mat = Bitmap._load_surf("gfx/grass4.png")
        dark = Bitmap._load_surf("gfx/darken.png")
        for x,h in enumerate(self.heights):
            if (skip % 3) == 0 and x+self.offset not in self.plateus:
                g = choice(grass)
                tmp = pygame.transform.rotozoom(g, -15+random()*30, 1)
                y = self.maxy-h - g.get_size()[1]*0.8 + 3 + random() * 3
                s.blit(tmp, (x,y+20))
            y = self.maxy-h
            s.blit(mat, (x,y+20+random()*3))
            s.blit(dark, (x,y+20))
            skip += 1
        self.bmp = Bitmap(s,hotspot=(0,0), border=True)
