
try:
    from psyco.classes import *
except ImportError:
    pass
    
import pygame
from pygame.locals import *

from pygext.gl.all import *
from pygext.gl.shapes.simple import rect
from pygext.math.vecalg import *

from steamer.config import Config
from steamer.world import *
from steamer.data import *
from steamer.land import Land, Blimp
from steamer.ship import Ship
from steamer.minimap import Minimap
from steamer.level import levels, tutorial, level_names

import time

class LevelStats(object):
    def __init__(self):
        self.start_time = time.time()
        self.end_time = None
        self.blimps = 0
        self.cannons = 0
        self.fuel = 0

class TitleScene(Scene):
    def enter(self, nofade=False):
        screen.set_clear_color((0,0,0,0))
        
        self.new_layer("bg")
        self.new_layer("text")
        self.new_layer("selector")
        bg = Entity("gfx/blueprint.jpg",hotspot=(0,0)).place("bg")
        bg.set(
            color=(250,250,150,0),
            y=0,
            x=0,
            scale=2,
            )
        if nofade:
            bg.alpha = 100
        else:
            bg.do(AlphaFade(100,secs=0.3))
        
        title = Entity(Bitmap("gfx/title.png")).place("text")
        title.set(
            centerx = 400,
            top = 50,
            alpha = 0,
            )
        if nofade:
            title.alpha = 255
        else:
            title.do(AlphaFade(255, secs=0.5))

        menu = Entity(Bitmap("gfx/menu.png")).place("text")
        menu.set(centerx = 400,
                 top = title.bottom + 100,
                 alpha = 0,
                 )
        if nofade:
            menu.alpha = 255
        else:
            menu.do(Delay(0.2) + AlphaFade(255,secs=0.5))

        self.selector = Entity("gfx/selection.png").place("selector")
        self.selector.set(
            color = (255,255,255,0),
            centerx = menu.centerx,
            top = menu.top-8,
            )
        if nofade:
            self.selector.alpha = 120
        else:
            self.selector.do(Delay(0.5) + AlphaFade(120, secs=0.5))

        if nofade:
            self.set_state("select")
        else:
            self.set_state("fadein")
            act = Delay(0.5) + SetState("select")
            act.do()

        self.select_pos = 0

        self.on = Entity(Bitmap("gfx/on.png")).place("text")
        self.off = Entity(Bitmap("gfx/off.png")).place("text")

        self.on.set(
            left = menu.right + 25,
            centery = menu.centery,
            )
        self.off.set(
            left = menu.right + 25,
            centery = menu.centery
            )
        self.off.do(Delay(0.2) + AlphaFade(255, secs=0.5))

        Config.reverse = World.data.reverse
        if Config.reverse:
            if not nofade:
                self.on.alpha = 0
                self.on.do(Delay(0.2) + AlphaFade(255, secs=0.5))
            self.off.hidden = True
        else:
            if not nofade:
                self.off.alpha = 0
                self.off.do(Delay(0.2) + AlphaFade(255, secs=0.5))
            self.on.hidden = True
            

    def handle_keydown(self, ev):
        if ev.key == K_ESCAPE:
            director.quit()

    def state_select_handle_keydown(self, ev):
        if ev.key == K_ESCAPE:
            director.quit()
        if ev.key == K_DOWN and self.select_pos < 2:
            self.set_state("movepos")
            self.selector.do(MoveDelta(0,56, secs=0.1) + SetState("select"))
            self.select_pos += 1
        elif ev.key == K_UP and self.select_pos > 0:
            self.set_state("movepos")
            self.select_pos -= 1
            self.selector.do(MoveDelta(0,-56, secs=0.1) + SetState("select"))
        elif ev.key in (K_RETURN, K_SPACE):
            self.select()

    def select(self):
        if self.select_pos == 0:
            director.set_scene(LevelSelect)
        elif self.select_pos == 1:
            Config.reverse = ~Config.reverse
            World.data.reverse = Config.reverse
            if Config.reverse:
                self.on.hidden = False
                self.off.hidden = True
            else:
                self.on.hidden = True
                self.off.hidden = False
        elif self.select_pos == 2:
            director.quit()

class LevelSelect(Scene):
    def enter(self):
        screen.set_clear_color((0,0,0,0))
        self.new_layer("bg")
        self.new_layer("text")
        self.new_layer("selector")
        self.new_layer("fader")
        
        bg = Entity("gfx/blueprint.jpg",hotspot=(0,0)).place("bg")
        bg.set(
            color=(250,250,150,100),
            y=0,
            x=0,
            scale=2,
            )

        self.font = GLFont(("serif", 32))

        self.selector = Entity("gfx/selection.png").place("selector")

        dat = World.data
        y = 50
        x = 100
        for lvl in range(0,11):
            e = TextEntity(self.font, level_names[lvl]).place("text")
            e.set(
                x = x,
                top = y,
                )
            if lvl in dat.unlocked:
                e.color = (200,200,0,255)
            else:
                e.color = (150,150,0,160)

            so = World.data.scores.get(lvl)
            if so is not None:
                e2 = TextEntity(self.font, "Hiscore: %s" % so.score).place("text")
                e2.set(left = 400,
                       top = y)
            y = e.bottom

        self.selector.set(
            centerx = e.centerx,
            y = 64,
            scale = 0.7,
            color = (255,255,255,100),
            )

        self.select_pos = 0

        self.set_state("select")

        f = self.fader = Entity(rect(0,0,800,600))
        f.place("fader").set(
            left = 0,
            top = 0,
            hidden = True,
            color = (0,0,0,0),
            )

    def state_select_handle_keydown(self, ev):
        if ev.key == K_ESCAPE:
            director.set_scene(TitleScene, True)
        elif ev.key == K_UP and self.select_pos > 0:
            self.set_state("movepos")
            self.selector.do(MoveDelta(0,-37, secs=0.1) + SetState("select"))
            self.select_pos -= 1
        elif ev.key == K_DOWN and self.select_pos < 11:
            self.set_state("movepos")
            self.selector.do(MoveDelta(0,37, secs=0.1) + SetState("select"))
            self.select_pos += 1
        elif ev.key in (K_SPACE,K_RETURN):
            if self.select_pos not in World.data.unlocked:
                return
            level = levels[self.select_pos]
            self.fader.hidden = False
            self.fader.do(AlphaFade(255, secs=0.5) + SetScene(MainScene,level))            

class MainScene(Scene):
    collengine = RadialCollisions
    
    def enter(self, level):
        self.new_layer("sky")
        self.new_layer("objects", camera=True)
        self.new_layer("ship", camera=True)
        self.new_layer("dirt", camera=True)
        self.new_static("static trees", camera=True)
        self.new_static("land", camera=True)
        self.new_static("land detail", camera=True)
        self.new_layer("steam", camera=True)
        self.new_static("gradient")
        self.new_static("ui")
        self.new_layer("meters")
        self.new_layer("glass")
        self.new_layer("text")
        self.new_layer("fader")

        self.fader = fader = Entity(rect(0,0,800,600)).place("fader")
        fader.set(
            top=0,left=0,
            color = (0,0,0,255),
            )
        fader.do(AlphaFade(0,secs=0.5) + Hide)

        screen.set_clear_color(None)

        World.init()
        World.level = level

        self.tutorial_index = 0        
        
        Entity("gfx/ui.png").set(left=0, bottom=600).place("ui")
        Entity("gfx/left_glass.png").set(
            left=0,
            bottom=600-20,
            alpha=50).place("glass")
        Entity("gfx/right_glass.png").set(
            right=775,
            bottom=600-20,
            alpha=50).place("glass")

        self.dial1 = Entity("gfx/dial_arrow.png",hotspot=(0.5,0.51)).set(
            x=58,
            y=517,
            alpha=200,
            angle=-145
            ).place("meters")

        self.dial2 = Entity("gfx/dial_arrow.png",hotspot=(0.5,0.51)).set(
            x=713,
            y=517,
            alpha=200,
            angle=-145
            ).place("meters")

        self.dial1turn = Rotate(0).do(self.dial1)
        self.dial2turn = Rotate(0).do(self.dial2)


        self.land = Land(level)
        World.land = self.land
        self.ship = Ship()

        self.minimap = Minimap(self.land)

        sh = Config.sky_height
        gr = GradientRect(800,400 + sh)
        gr.set_colors(
            top=(50,50,150,255),
            bottom=(255,255,255,255),
            )
        self.sky = Entity(gr).place("sky").set(
            y=-sh,
            x=0,
            )

        self.max_scroll = len(self.land.segments)*128-800

        if level is levels[0]:
            self.tutorial_text = TextEntity(World.font,"").set(
                color = (255,255,255,0),
                ).place("text")
            self.show_tutorial_text()

        World.stats = LevelStats()

        self.victory = False

    def show_tutorial_text(self):
        txt = tutorial[self.tutorial_index]
        te = self.tutorial_text
        te.set_text(txt)
        te.do(AlphaFade(255,secs=0.5) + Delay(3) + AlphaFade(0,secs=0.5))
        te.centerx = 400
        te.top = 15
        self.tutorial_index += 1
        if self.tutorial_index < len(tutorial):
            act = Delay(4.2) + CallFunc(self.show_tutorial_text)
            act.do()


    def collision_ship_tree(self, ship, tree):
        if ship.move.vx > 0:
            sgn = 1
        else:
            sgn = -1
        tree.fall(sgn)
        ship.damage(5)

    def collision_bullet_tree(self, bullet, tree):
        if bullet.move.vx > 0:
            sgn = 1
        else:
            sgn = -1
        tree.fall(sgn)

    def collision_bullet_cannon(self, bullet, cannon):
        if cannon.dead:
            return
        if bullet.node is self.ship.cannon_node:
            World.stats.cannons += 1
        bullet.delete()
        cannon.boom()

    def collision_bullet_blimp(self, bullet, blimp):
        if blimp.dead:
            return
        if bullet.node is self.ship.cannon_node:
            World.stats.blimps += 1
        blimp.boom()
        bullet.delete()

    def collision_ship_blimp(self, ship, blimp):
        if blimp.dead:
            return
        ship.damage(20)
        ship.move.mul_velocity(0.4, 0.4)
        v = vec(ship.x-blimp.x,ship.y-blimp.y)
        a = direction(v)
        dx,dy = sincos(a, 200)
        ship.move.add_velocity(dx,dy)
        blimp.boom()

    def collision_ship_bullet(self, ship, bullet):
        World.smoke_system.new_emitter(CannonEmitter, bullet)
        World.fire_system.new_emitter(ExplosionEmitter, bullet)
        ship.damage(30)
        ship.move.vx += 0.5 * bullet.move.vx
        ship.move.vy += 0.5 * bullet.move.vy
        bullet.clear_collnodes()
        bullet.do(Delay(0.1)+Delete)

        v = vec(ship.x-bullet.x,ship.y-bullet.y)
        a = direction(v)
        dx,dy = sincos(a, 100)

    def collision_ship_cannon(self, ship, cannon):
        ship.damage(10)
        ship.move.vy = ship.move.vy * 0.3 - 150

    def realtick(self):
        self.ship.tick()
        if World.level is levels[0]:
            self.ship.fuel = 100
            if self.ship.pressure < 20:
                self.ship.pressure = 20

        self.read_input()

        fc = self.dial2.angle
        if fc > 180:
            fc -= 360
        ft = self.ship.fuel * 2.65 - 145
        if fc < ft:
            self.dial2turn.set_velocity(50)
        else:
            self.dial2turn.set_velocity(-50)

        pc = self.dial1.angle
        if pc > 180:
            pc -= 360
        pt = self.ship.pressure * 2.85 - 145
        if pc < pt:
            self.dial1turn.set_velocity(50)
        else:
            self.dial1turn.set_velocity(-50)

        for c in World.land.cannons:
            c.tick(self.ship)

        if self.ship.crashed:
            pass

        if self.ship.landed is World.land.pads[1] and World.level is not levels[0] and not self.victory:
            self.victory = True
            stats = World.stats
            stats.end_time = time.time()
            stats.fuel = self.ship.fuel
            self.fader.do(Delay(1) + Show + AlphaFade(255,secs=1) + SetScene(Scoring))

    def tick(self, cfg=Config):
        sh =  cfg.sky_height
        cx = self.ship.x - 400
        cy = self.ship.y - 200
        if cy < -sh:
            cy = -sh
        elif cy > 0:
            cy = 0
        if cx < 0:
            cx = 0
        if cx > self.max_scroll:
            cx = self.max_scroll
            
        self.offset = (cx,cy)
        self.minimap.set(self.ship)

        self.sky.y = (sh + cy) * -0.5
        

    def read_input(self, cfg=Config):
        if self.ship.crashed or self.victory:
            return
        k = pygame.key.get_pressed()
        rev = World.data.reverse
        if self.ship.pressure > 0:
            if k[cfg.key_left_thrust]:
                if rev:
                    self.ship.thrust_right()
                else:
                    self.ship.thrust_left()
            if k[cfg.key_right_thrust]:
                if rev:
                    self.ship.thrust_left()
                else:
                    self.ship.thrust_right()
            if k[cfg.key_brake]:
                self.ship.thrust_mid()
            if k[cfg.key_cannon]:
                self.ship.fire_cannon()
        if k[cfg.key_burn]:
            self.ship.burn()

    def handle_keydown(self, ev):
        if ev.key == K_ESCAPE:
            self.fader.do(Show + AlphaFade(255,secs=0.5) + SetScene(LevelSelect))


class Scoring(Scene):
    def enter(self):
        screen.set_clear_color((0,0,0,0))
        self.new_layer("bg")
        self.new_layer("text")
        self.new_layer("fader")
        
        bg = Entity("gfx/blueprint.jpg",hotspot=(0,0)).place("bg")
        bg.set(
            color=(250,250,150,100),
            y=0,
            x=0,
            scale=2,
            )

        self.font = GLFont(("serif", 32))

        self.fader = f = Entity(rect(0,0,800,600)).place("fader")
        f.set(
            top=0,left=0,
            color = (0,0,0,255),
            )
        f.do(AlphaFade(0,secs=0.5) + Hide)

        stats = World.stats

        t = stats.end_time - stats.start_time
        score = 0
        time_bonus = 100000.0 / (t+20.0)
        time_bonus = int(time_bonus//500) * 500

        fuel_bonus = int(stats.fuel) * 500

        blimp_bonus = stats.blimps * 500
        cannon_bonus = stats.cannons * 5000

        score = time_bonus + fuel_bonus + blimp_bonus + cannon_bonus

        lines = [
            ("Time Score", time_bonus),
            ("Fuel Score", fuel_bonus),
            (None,None),
            ("Blimp Kill Bonus", blimp_bonus),
            ("Cannon Kill Bonus", cannon_bonus),
            (None,None),
            (None,None),
            ("Total Score", score),
            ]

        t = 0
        y = 100
        for l,pts in lines:
            if l is not None:
                e1 = TextEntity(self.font, l).place("text")
                e1.set(left = 50,
                       top = y,
                       )
                e2 = TextEntity(self.font, str(pts)).place("text")
                e2.set(left = 400,
                       top = y,
                       )
            y += 40

        ul = World.level.index + 1
        if ul not in World.data.unlocked:
            World.data.unlocked.append(ul)
        so = Score(score, stats.fuel, t)
        old = World.data.scores.get(World.level.index)
        if old is None or old.score < so.score:
            World.data.scores[World.level.index] = so

    def handle_keydown(self, ev):
        self.fader.do(Show + AlphaFade(255,secs=0.5) + SetScene(LevelSelect))

def run():
##    import sys
##    if len(sys.argv) > 1:
##        level = levels[int(sys.argv[1])]
##    else:
##        level = levels[10]
    screen.init((800,600),title="Steamer!",fullscreen=Config.fullscreen)
##    director.run(MainScene, level)
    World.data = load_data()
    director.run(TitleScene)
    save_data(World.data)
