
try:
    from psyco.classes import *
except ImportError:
    pass


from pygext.gl.all import *
from pygext.math.vecalg import sincos
from pygext.render.stencils import alpha_stencil
from pygext.lazy import Random

from steamer.config import Config
from steamer.world import *

            

class Ship(EntityNode):
    image = "gfx/ship.png"
    layer = "ship"

    def init(self):
        s = Bitmap._load_surf(self.image)
        s = alpha_stencil(s)
        s = Entity(s)
        s.attach_to(self)
        s.alpha = 0

        self.stencil = s

        l = self.left_thruster = Entity()
        l.set(x=-18, y = 8)
        l.attach_to(self)
        r = self.right_thruster = Entity()
        r.set(x=18,y=8)
        r.attach_to(self)

        s = World.steam_system

        self.left_emitter = SteamEmitter(s,l)
        self.right_emitter = SteamEmitter(s,r)
        self.center_emitter = SteamEmitter2(s,self)

        c = self.cannon_node = Entity()
        c.set(x=5, y=-18)
        c.attach_to(self)

        self.move = Move(0,0).do(self)
        self.turn = Rotate(0).do(self)

        self.fuel = Config.start_fuel
        self.pressure = Config.start_pressure

        self.ticker = 0
        self.crashed = False

        self.maxx = len(World.land.segments)*128 - 15

        self.cannon_recharging = False

        self.add_collnode("ship", 18)
        self.landed = True
        pad = World.land.pads[0]
        self.centerx = pad.centerx
        self.bottom = pad.top

    def flash(self):
        self.stencil.abort_actions(ColorFade)
        self.stencil.color = (255,100,100,150)
        self.stencil.do(ColorFade((50,0,0,0), secs=0.5))

    def damage(self, pts):
        self.flash()
        World.steam_system.new_emitter(SteamEmitter2, self)
        self.pressure -= min(self.pressure,pts)

    def tick(self, cfg=Config):        
        if self.landed:
            self.move.set_velocity(0,0)
            if self.pressure <= 0:
                self.crashed = True
            return
        if self.crashed:
            self.move.mul_velocity(0.8,0.8)
            self.turn.mul_velocity(0.8)
            return
        land = World.land
        self.move.add_velocity(0,cfg.gravity)
        self.move.mul_velocity(cfg.ship_friction,cfg.ship_friction)
        self.turn.mul_velocity(cfg.ship_turn_friction)
        x = int(self.x)
        h1 = land.get_h(x)
        h2 = land.get_h(x-13)-5
        h3 = land.get_h(x+13)-5        
        h,crashx = max((h1,x),(h2,x-13),(h3,x+13))
        d = self.y-(585-h)
        vx = self.move.vx
        on_pad = False
        for pad in land.pads:
            if abs(self.x-pad.x) < 30:
                on_pad = True
                break
        if on_pad:
            d += 8
        if d > 10:            
            self.move.vx = 0
        if d > 0:
            if not on_pad:
                CrashEmitter.direction = vx*0.2
                World.dirt_system.new_emitter(CrashEmitter, x=crashx, y=620-h)
            vel = abs(self.move.vx)*2 + abs(self.move.vy)
            a = self.angle % 360
            if a > 180:
                a -= 360
            if not on_pad or (vel > 100 or abs(a) > 15):
                self.damage(10)
                if self.pressure <= 0:
                    self.crashed = True
                    self.move.vy *= 0.3
                    World.dirt_system.new_emitter(CrashEmitter, x=crashx, y=620-h)
                    World.smoke_system.new_emitter(SmokeEmitter, self)
                elif self.move.vy > 0:
                    self.move.vy = self.move.vy * -0.5
                    self.move.vy -= 80
            else:
                self.move.set_velocity(0,0)
                self.angle = 0
                self.turn.set_velocity(0)
                self.bottom = pad.top
                self.landed = pad
        else:
            a = self.angle % 360
            if a < 100:
                self.turn.add_velocity(3-abs(a-90)/30.0)
            elif a > 260:
                self.turn.add_velocity(-3+abs(a-270)/30.0)
            
        if self.x < 15 and self.move.vx < 0:
            self.move.vx = 0
            self.x = 15
        if self.x > self.maxx and self.move.vx > 0:
            self.move.vx = 0
            self.x = self.maxx
        miny = -cfg.sky_height + 20
        if self.y < miny and self.move.vy < 0:
            self.move.vy = 0
            self.y = miny

        if self.fuel > 0:
            self.fuel -= cfg.fuel_consumption
            self.pressure += cfg.fuel_efficiency
            if not self.ticker % 5:
                self.center_emitter.emit()

        self.ticker += 1            

    def thrust_left(self, cfg=Config):
        pow = min(self.pressure / cfg.pressure_power_limit, 1.0)
        self.turn.add_velocity(cfg.turn_rate*pow)
        dx,dy = sincos(self.angle+10,cfg.thrust*pow)
        self.move.add_velocity(dx,dy)
        self.left_thruster.angle = self.angle
        self.left_emitter.color = (255,255,255,255*pow*pow)
        self.left_emitter.emit()
        self.pressure -= cfg.pressure_consumption * pow
        self.landed = False

    def thrust_right(self, cfg=Config):
        pow = min(self.pressure / cfg.pressure_power_limit, 1.0)
        self.turn.add_velocity(-cfg.turn_rate*pow)
        dx,dy = sincos(self.angle-10, cfg.thrust*pow)
        self.move.add_velocity(dx,dy)
        self.right_thruster.angle = self.angle
        self.right_emitter.color = (255,255,255,255*pow*pow)
        self.right_emitter.emit()
        self.pressure -= cfg.pressure_consumption * pow
        self.landed = False

    def thrust_mid(self,cfg=Config):
        self.center_emitter.emit()
        self.move.mul_velocity(cfg.break_efficiency, cfg.break_efficiency)
        self.turn.mul_velocity(0.95)
        self.pressure -= cfg.break_consumption

    def burn(self, cfg=Config):
        if self.fuel <= 0:
            return
        if not self.ticker % 2:
            self.center_emitter.emit()
        self.pressure += cfg.burn_pressure
        self.fuel -= cfg.burn_fuel
        self.center_emitter.emit()

    def fire_cannon(self, cfg=Config):
        if self.cannon_recharging or self.pressure < cfg.cannon_pressure:
            return
        self.pressure -= cfg.cannon_pressure
        self.do(Delay(1) + SetAttr(cannon_recharging=False))
        self.cannon_recharging = True
        self.cannon_node.angle = self.angle
        World.steam_system.new_emitter(CannonEmitter, self.cannon_node)
        if not self.landed:
            dx,dy = sincos(self.angle, -cfg.cannon_kickback)
            self.move.add_velocity(dx,dy)

        b = Bullet(node=self.cannon_node)
